//+=============================================================================
//
// file :         IntlkGun.cpp
//
// description :  C++ source for the IntlkGun and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                IntlkGun are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: graziano $
//
// $Revision: 1.12 $
//
// $Log: IntlkGun.cpp,v $
// Revision 1.12  2019-12-23 10:29:44  graziano
// Fix realfiltimer to unsigned
//
// Revision 1.11  2019-01-07 15:21:34  graziano
// fixed for upgrade of plc interlock booster
//
// Revision 1.10  2017-03-30 11:34:28  claudio
// ticket 1934 porting to tango-9
//
// Revision 1.9  2014-07-29 12:23:19  claudio
// removed lots of cout, use ERROR_STREAM
//
// Revision 1.8  2013-08-26 10:35:52  claudio
// Device_4Impl for tango-8
//
// Revision 1.7  2008-09-02 10:29:50  sara
// improver attribute read from lower level srv
//
// Revision 1.6  2008/09/01 08:16:45  sara
// fixed bug -delete- in always_executed_hook
//
// Revision 1.5  2008/08/11 10:29:15  sara
// cached attributes
//
// Revision 1.4  2008/02/06 11:07:01  sara
// Gun Fil.Timer correction
//
// Revision 1.3  2008/02/01 14:20:21  sara
// added RealFilTimer attribute for gun current timer bar
//
// Revision 1.2  2007/12/06 14:26:37  sara
// Alarm attributes corresponding to PLC outputs take their value from
// the state of the outputs
//
// Revision 1.1.1.1  2007/08/31 16:24:36  sara
// server for gun interlock
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name   |  Method's name
//	----------------------------------------
//  State            |  dev_state()
//  Status           |  dev_status()
//  Acknowledge_all  |  acknowledge_all()
//
//===================================================================


#include <tango.h>
#include <IntlkGun.h>
#include <IntlkGunClass.h>
//-------------------------------
static char*  GunSafetyMask[] = {
	"Gun _VAC_OK_SLOW",
	"Gun_I_Fil<MAX",
	"Gun_safety_key",
	"Gun dr closed",
	"Gun safety foil",
	"Gun earth",
	"Gun air flux",
	"Gun fuse"
};
static char* FilAbiMask[] = {
	"Abi Fil Gun",
	"I_Fil_Gun_OK"
};
static char* FilCurrMask[] = {
	"Start_I_Fil_timer"
};
static char* FilReadyMask[] = {
	"I_Fil_Gun_ready",
	"H.V.ABI",
	"H.V.ABI",
	"RF_ALR_SIP40_P 1.1",
	"Gun_V_GRILL>MIN",
	"Catode 80 V"
};
static char* GunHVMask[] = {
	"Gun HV abi",
	"Gun HV OK",
	"Gun_ABI_VLV",
	"RF_ALR_SIP40_P 1.2",
	"RF_ALR_SIP40_P 1.3",
	"RF_ALR_SIP40_P 1.4",
	"RF_ALR_SIP300_P 1.1",
	"RF_ALR_SIP300_P 1.2",
	"RF_ALR_SIP300_P 1.3",
	"RF_ALR_SIP300_P 1.4"
};
static char* AbiTrigGunMask[] = {
	"Gun trigger abi"
};
//-------------------------------
namespace IntlkGun_ns
{

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::IntlkGun(string &s)
// 
// description : 	constructor for simulated IntlkGun
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
IntlkGun::IntlkGun(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

IntlkGun::IntlkGun(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

IntlkGun::IntlkGun(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void IntlkGun::delete_device()
{
	//	Delete device's allocated object
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void IntlkGun::init_device()
{
	INFO_STREAM << "IntlkGun::IntlkGun() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------

	get_device_property();

	// connect to interlock device 
	devintlk = new Tango::DeviceProxy(intlk_srv);
	
	timestamp_50 = 0;	
	timestamp_53 = 0;	
	curr_fil_val = 0;
	grid_val = 0;
	fil_timer = 0;
	real_fil_timer = 0;

	// send command 
	try {
		if (devintlk->ping()) {
			set_state(Tango::ON);	
			set_status("Connected to device " + intlk_srv);
			ERROR_STREAM << "IntlkGun::IntlkGun() Connected to devintlk:"   << intlk_srv   << endl;
		}	
	} 
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Failed to connect to devintlk " + intlk_srv);
		ERROR_STREAM <<"IntlkGun::IntlkGun() Failed to connect to devintlk " + intlk_srv << endl;
	}
	//init position, names of the signals
	vector <string> gun_saf, fl_ab, fl_crr, fl_rdy, gun_hv, abi_trg_gn;
	for(int k=0;k<GUNSAFETY_DIM;k++){
		pos_gun_safety[k] = -1;
		name_gun_safety[k] = GunSafetyMask[k];
		pos_alm_gun_safety[k] = -1;
		alm_gun_safety[k] = true;
		name_alm_gun_safety[k] = GunSafetyMask[k];
		gun_saf.push_back(string(name_gun_safety[k]));
	}
	for(int k=0;k<FILABI_DIM;k++){
		pos_fil_abi[k] = -1;
		name_fil_abi[k] = FilAbiMask[k];
		alm_fil_abi[k] = true;
		name_alm_fil_abi[k] = FilAbiMask[k];
		fl_ab.push_back(string(name_fil_abi[k]));
	}
	for(int k=0;k<FILCURR_DIM;k++){
		pos_fil_curr[k] = -1;
		name_fil_curr[k] = FilCurrMask[k];
		pos_alm_fil_curr[k] = -1;
		alm_fil_curr[k] = true;
		name_alm_fil_curr[k] = FilCurrMask[k];
		fl_crr.push_back(string(name_fil_curr[k]));
	}
	for(int k=0;k<FILREADY_DIM;k++){
		pos_fil_ready[k] = -1;
		name_fil_ready[k] = FilReadyMask[k];
		pos_alm_fil_ready[k] = -1;
		alm_fil_ready[k] = true;
		name_alm_fil_ready[k] = FilReadyMask[k];
		fl_rdy.push_back(string(name_fil_ready[k]));
	}
	for(int k=0;k<GUNHV_DIM;k++){
		pos_gun_hv[k] = -1;
		name_gun_hv[k] = GunHVMask[k];
		pos_alm_gun_hv[k] = -1;
		alm_gun_hv[k] = true;
		name_alm_gun_hv[k] = GunHVMask[k];
		gun_hv.push_back(string(name_gun_hv[k]));
	}
	for(int k=0;k<ABITRIGGUN_DIM;k++){
		pos_abi_trig_gun[k] = -1;
		name_abi_trig_gun[k] = AbiTrigGunMask[k];
		pos_alm_abi_trig_gun[k] = -1;
		alm_abi_trig_gun[k] = true;
		name_alm_abi_trig_gun[k] = AbiTrigGunMask[k];
		abi_trg_gn.push_back(string(name_abi_trig_gun[k]));
	}
        Tango::Database *db = new Tango::Database();
        Tango::DbData db_gun_saf, db_fl_ab, db_fl_crr, db_fl_rdy, db_gun_hv, db_abi_trg_gn;

	Tango::DbDatum GunSafety("gunSafety"), sig_n1("signal_names");
	GunSafety << (short int) 1;
	db_gun_saf.push_back(GunSafety);
	sig_n1 << gun_saf;
	db_gun_saf.push_back(sig_n1);

	Tango::DbDatum FilAbi("filAbi"), sig_n2("signal_names");
	FilAbi << (short int) 1;
	db_fl_ab.push_back(FilAbi);
	sig_n2 << fl_ab;
	db_fl_ab.push_back(sig_n2);

	Tango::DbDatum FilCurr("filCurr"), sig_n3("signal_names");
	FilCurr << (short int) 1;
	db_fl_crr.push_back(FilCurr);
	sig_n3 << fl_crr;
	db_fl_crr.push_back(sig_n3);

	Tango::DbDatum FilReady("filReady"), sig_n4("signal_names");
	FilReady << (short int) 1;
	db_fl_rdy.push_back(FilReady);
	sig_n4 << fl_rdy;
	db_fl_rdy.push_back(sig_n4);
 
	Tango::DbDatum GunHV("gunHV"), sig_n5("signal_names");
	GunHV << (short int) 1;
	db_gun_hv.push_back(GunHV);
	sig_n5 << gun_hv;
	db_gun_hv.push_back(sig_n5);

	Tango::DbDatum AbiTrigGun("abiTrigGun"), sig_n6("signal_names");
	AbiTrigGun << (short int) 1;
	db_abi_trg_gn.push_back(AbiTrigGun);
	sig_n6 << abi_trg_gn;
	db_abi_trg_gn.push_back(sig_n6);

	//makes -signal_names- visible through attribute property 
	try {
		db->set_timeout_millis(10000);	
		db->put_device_attribute_property(get_name(), db_gun_saf);
		db->put_device_attribute_property(get_name(), db_fl_ab);
		db->put_device_attribute_property(get_name(), db_fl_crr);
		db->put_device_attribute_property(get_name(), db_fl_rdy);
		db->put_device_attribute_property(get_name(), db_gun_hv);
		db->put_device_attribute_property(get_name(), db_abi_trg_gn);
	} catch (Tango::DevFailed &e) {
		Tango::Except::print_exception(e);
	}
	//check if signals in the Masks are available from the low-level server
	match_labels("labels_50");
	match_labels("labels_53");
}


//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void IntlkGun::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Intlk_srv"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	IntlkGunClass	*ds_class =
		(static_cast<IntlkGunClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize Intlk_srv from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  intlk_srv;
	//	Try to initialize Intlk_srv from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  intlk_srv;
	//	And try to extract Intlk_srv value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  intlk_srv;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void IntlkGun::always_executed_hook()
{
	
	struct timeval now;
	vector<Tango::DeviceAttribute> *devattr_states = 0;
	vector<Tango::DeviceAttribute> *devattr_alarm = 0;
	vector<string> attr_names_states;
	vector<string> attr_names_alarm;
	vector<double> raw_val;
	vector<bool> values53;
	vector<bool> values50;
	
	gettimeofday(&now, NULL);

	if (abs((now.tv_sec - last_cache_update.tv_sec)*1000000+ 
		   (now.tv_usec - last_cache_update.tv_usec)) > RFSH_CACHE) {
		try {
		
			attr_names_states.push_back("db10");
			attr_names_states.push_back("db_50");
			devattr_states = devintlk->read_attributes(attr_names_states);
			(*devattr_states)[0] >> raw_val;
			(*devattr_states)[1] >> values50;
			delete devattr_states;
			for(size_t i=0;i<values50.size() && i<MESG_50_LEN_B;i++){
				msg_50[i] = (Tango::DevBoolean)(values50.at(i));
			}
			for(size_t i=0;i<raw_val.size() && i<RAW_MSG_10_LEN;i++){
				rawmsg_10[i] = (Tango::DevDouble)(raw_val.at(i));
			}
		
			gettimeofday(&last_cache_update,NULL);
		}
		catch(Tango::DevFailed &e) {
			ERROR_STREAM <<"attribute refresh db_50 failed at "<< now.tv_sec<<"/"<< now.tv_usec<<endl;
			if(devattr_states)
				delete devattr_states;
		}
		try {
		
			attr_names_alarm.push_back("rt_db_53");
			devattr_alarm = devintlk->read_attributes(attr_names_alarm);
			(*devattr_alarm)[0] >> values53;
			delete devattr_alarm;
			for(size_t i=0;i<values53.size() && i<MESG_53_LEN_B;i++){
				msg_53[i] = (Tango::DevBoolean)(values53.at(i));
			}
		
			gettimeofday(&last_cache_update,NULL);
		}
		catch(Tango::DevFailed &e) {
			ERROR_STREAM << "attribute refresh rt_db_53 failed at "<< now.tv_sec<<"/"<< now.tv_usec<<endl;
			if(devattr_alarm)
				delete devattr_alarm;
		}
	}

}
//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "IntlkGun::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_RealFilTimer
// 
// description : 	Extract real attribute values for RealFilTimer acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_RealFilTimer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_RealFilTimer(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<u_short> raw_val;

	attr_names.push_back("raw_msg_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> raw_val;
	delete devattr;
*/
#if 0
	unsigned char arr[2]; 
	for(int j=0, k=1;j<2;j++, k--){
//		DEBUG_STREAM << j+294 << "-"<< ((Tango::DevUShort)(raw_val.at(j+294))) << endl;
		DEBUG_STREAM << j+294 << "-"<< rawmsg_50[j+294] << endl;
//		bcopy(&(raw_val.at(j+294)), &(arr[k]), 1); 
		bcopy(&(rawmsg_50[j+294]), &(arr[k]), 1); 
	}
	u_short timer = 0;

	bcopy(&arr[0], &timer, 2);
	timer = 0xffff-timer;
#else
	unsigned short timer =  (unsigned short)rawmsg_10[0];
	if(timer == (unsigned short)(-1))
		timer = 0;
#endif
	real_fil_timer = (float)timer;
	real_fil_timer /= 60;// div 600 mul 10

	attr_RealFilTimer_read = &real_fil_timer;

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_RealFilTimer_read, taim, Tango::ATTR_VALID, 1,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_FilTimer_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_FilTimer
// 
// description : 	Extract real attribute values for FilTimer acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_FilTimer(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_FilTimer(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<u_short> raw_val;

	attr_names.push_back("raw_msg_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> raw_val;
	delete devattr;
*/
#if 0
	unsigned char arr[4]; 
	for(int j=0, k=3;j<4;j++, k--){
		
//		DEBUG_STREAM << j+292 << "-"<< ((Tango::DevUShort)(raw_val.at(j+292))) << "/";
		DEBUG_STREAM << j+292 << "-"<< rawmsg_50[j+292] << "/";
//		bcopy(&(raw_val.at(j+292)), &(arr[k]), 1); 
		bcopy(&(rawmsg_50[j+292]), &(arr[k]), 1); 
	}
	u_short counter = 0;

	bcopy(&arr[0], &counter, 2);
	DEBUG_STREAM <<"counter:"<< counter << endl;
	fil_timer = (u_short)counter;
#else
	double fil_timer_d = rawmsg_10[0];
	fil_timer = (u_short)fil_timer_d;
#endif

	attr_FilTimer_read = &fil_timer;

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_FilTimer_read, taim, Tango::ATTR_VALID, 1,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_FilTimer_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_AlmAbiTrigGun
// 
// description : 	Extract real attribute values for AlmAbiTrigGun acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_AlmAbiTrigGun(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_AlmAbiTrigGun(Tango::Attribute &attr) entering... "<< endl;

/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values50;
	vector<bool> values53;

	attr_names.push_back("rt_db_53");
	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values53;
	(*devattr)[1] >> values50;
	delete devattr;
*/

	for(int j=0;j<ABITRIGGUN_DIM;j++){
		if(j==0){
//			alm_abi_trig_gun[j] = ((Tango::DevBoolean)(values50.at(pos_abi_trig_gun[j])));
			alm_abi_trig_gun[j] = msg_50[pos_abi_trig_gun[j]];
			DEBUG_STREAM << "read values " << name_alm_abi_trig_gun[j]<< " val: "<< alm_abi_trig_gun[j]<<endl;
		}
		else{
//			alm_abi_trig_gun[j] = ((Tango::DevBoolean)(values53.at(pos_alm_abi_trig_gun[j])));
			alm_abi_trig_gun[j] = msg_53[pos_alm_abi_trig_gun[j]];
			DEBUG_STREAM << "read values " << name_alm_abi_trig_gun[j]<< " val: "<< alm_abi_trig_gun[j]<<endl;
		}
	}
	
	attr_AlmAbiTrigGun_read = &alm_abi_trig_gun[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_53 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_AlmAbiTrigGun_read, taim, Tango::ATTR_VALID, ABITRIGGUN_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_AlmGunSafety
// 
// description : 	Extract real attribute values for AlmGunSafety acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_AlmGunSafety(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_AlmGunSafety(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values;

	attr_names.push_back("rt_db_53");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values;
	delete devattr;
*/

	for(int j=0;j<GUNSAFETY_DIM;j++){
//		alm_gun_safety[j] = ((Tango::DevBoolean)(values.at(pos_alm_gun_safety[j])));
		alm_gun_safety[j] = msg_53[pos_alm_gun_safety[j]];
		DEBUG_STREAM << "read values " << name_alm_gun_safety[j]<< " val: "<< alm_gun_safety[j]<<endl;
	}
	
	attr_AlmGunSafety_read = &alm_gun_safety[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_53 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_AlmGunSafety_read, taim, Tango::ATTR_VALID, GUNSAFETY_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}

}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::write_AlmGunSafety
// 
// description : 	Write AlmGunSafety attribute values to hardware.
//
//-----------------------------------------------------------------------------
void IntlkGun::write_AlmGunSafety(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "IntlkGun::write_AlmGunSafety(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_AlmFilAbi
// 
// description : 	Extract real attribute values for AlmFilAbi acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_AlmFilAbi(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_AlmFilAbi(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values50;
	vector<bool> values53;

	attr_names.push_back("rt_db_53");
	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values53;
	(*devattr)[1] >> values50;
	delete devattr;
*/


	for(int j=0;j<FILABI_DIM;j++){
		if(j==0){
//			alm_fil_abi[j] = ((Tango::DevBoolean)(values50.at(pos_fil_abi[j])));
			alm_fil_abi[j] = msg_50[pos_fil_abi[j]];
			DEBUG_STREAM << "read values " << name_alm_fil_abi[j]<< " val: "<< alm_fil_abi[j]<<endl;
		}
		else{
//			alm_fil_abi[j] = ((Tango::DevBoolean)(values53.at(pos_alm_fil_abi[j])));
			alm_fil_abi[j] = msg_53[pos_alm_fil_abi[j]];
			DEBUG_STREAM << "read values " << name_alm_fil_abi[j]<< " val: "<< alm_fil_abi[j]<<endl;
		}
	}
	
	attr_AlmFilAbi_read = &alm_fil_abi[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_53 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_AlmFilAbi_read, taim, Tango::ATTR_VALID, FILABI_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::write_AlmFilAbi
// 
// description : 	Write AlmFilAbi attribute values to hardware.
//
//-----------------------------------------------------------------------------
void IntlkGun::write_AlmFilAbi(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "IntlkGun::write_AlmFilAbi(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_AlmFilCurr
// 
// description : 	Extract real attribute values for AlmFilCurr acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_AlmFilCurr(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_AlmFilCurr(Tango::Attribute &attr) entering... "<< endl;

/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values50;
	vector<bool> values53;

	attr_names.push_back("rt_db_53");
	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values53;
	(*devattr)[1] >> values50;
	delete devattr;
*/

	for(int j=0;j<FILCURR_DIM;j++){
		if(j==0){
//			alm_fil_curr[j] = ((Tango::DevBoolean)(values50.at(pos_fil_curr[j])));
			alm_fil_curr[j] = msg_50[pos_fil_curr[j]];
			DEBUG_STREAM << "read values " << name_alm_fil_curr[j]<< " val: "<< alm_fil_curr[j]<<endl;
		}
		else{
//			alm_fil_curr[j] = ((Tango::DevBoolean)(values53.at(pos_alm_fil_curr[j])));
			alm_fil_curr[j] = msg_53[pos_alm_fil_curr[j]];
			DEBUG_STREAM << "read values " << name_alm_fil_curr[j]<< " val: "<< alm_fil_curr[j]<<endl;
		}
	}
	
	attr_AlmFilCurr_read = &alm_fil_curr[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_53 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_AlmFilCurr_read, taim, Tango::ATTR_VALID, FILCURR_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::write_AlmFilCurr
// 
// description : 	Write AlmFilCurr attribute values to hardware.
//
//-----------------------------------------------------------------------------
void IntlkGun::write_AlmFilCurr(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "IntlkGun::write_AlmFilCurr(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_AlmFilReady
// 
// description : 	Extract real attribute values for AlmFilReady acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_AlmFilReady(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_AlmFilReady(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values50;
	vector<bool> values53;

	attr_names.push_back("rt_db_53");
	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values53;
	(*devattr)[1] >> values50;
	delete devattr;
*/
	for(int j=0;j<FILREADY_DIM;j++){
		if(j==0){
//			alm_fil_ready[j] = ((Tango::DevBoolean)(values50.at(pos_fil_ready[j])));
			alm_fil_ready[j] = msg_50[pos_fil_ready[j]];
			DEBUG_STREAM << "read values " << name_alm_fil_ready[j]<< " val: "<< alm_fil_ready[j]<<endl;
		}
		else{
//			alm_fil_ready[j] = ((Tango::DevBoolean)(values53.at(pos_alm_fil_ready[j])));
			alm_fil_ready[j] = msg_53[pos_alm_fil_ready[j]];
			DEBUG_STREAM << "read values " << name_alm_fil_ready[j]<< " val: "<< alm_fil_ready[j]<<endl;
		}
	}
	
	attr_AlmFilReady_read = &alm_fil_ready[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_53 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_AlmFilReady_read, taim, Tango::ATTR_VALID, FILREADY_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::write_AlmFilReady
// 
// description : 	Write AlmFilReady attribute values to hardware.
//
//-----------------------------------------------------------------------------
void IntlkGun::write_AlmFilReady(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "IntlkGun::write_AlmFilReady(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_AlmGunHV
// 
// description : 	Extract real attribute values for AlmGunHV acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_AlmGunHV(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_AlmGunHV(Tango::Attribute &attr) entering... "<< endl;

/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values50;
	vector<bool> values53;

	attr_names.push_back("rt_db_53");
	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values53;
	(*devattr)[1] >> values50;
	delete devattr;
*/

	for(int j=0;j<GUNHV_DIM;j++){
		if(j==0){
//			alm_gun_hv[j] = ((Tango::DevBoolean)(values50.at(pos_gun_hv[j])));
			alm_gun_hv[j] = msg_50[pos_gun_hv[j]];
			DEBUG_STREAM << "read values " << name_alm_gun_hv[j]<< " val: "<< alm_gun_hv[j]<<endl;
		}
		else{
//			alm_gun_hv[j] = ((Tango::DevBoolean)(values53.at(pos_alm_gun_hv[j])));
			alm_gun_hv[j] = msg_53[pos_alm_gun_hv[j]];
			DEBUG_STREAM << "read values " << name_alm_gun_hv[j]<< " val: "<< alm_gun_hv[j]<<endl;
		}
	}
	
	attr_AlmGunHV_read = &alm_gun_hv[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_53 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_AlmGunHV_read, taim, Tango::ATTR_VALID, GUNHV_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::write_AlmGunHV
// 
// description : 	Write AlmGunHV attribute values to hardware.
//
//-----------------------------------------------------------------------------
void IntlkGun::write_AlmGunHV(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "IntlkGun::write_AlmGunHV(Tango::WAttribute &attr) entering... "<< endl;
}


//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_CurrFilVal
// 
// description : 	Extract real attribute values for CurrFilVal acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_CurrFilVal(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_CurrFilVal(Tango::Attribute &attr) entering... "<< endl;
#if 0
	unsigned char arr[4]; 
	for(int j=0, k=3;j<4;j++, k--){
		bcopy(&(rawmsg_50[j+296]), &(arr[k]), 1); 
	}
	float curr;
	bcopy(&arr[0], &curr, 4);
	DEBUG_STREAM <<"curr:"<< curr << endl;
	curr_fil_val = (double)curr;
#else
	curr_fil_val = rawmsg_10[2];
#endif
	attr_CurrFilVal_read = &curr_fil_val;

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;
		attr.set_value_date_quality(attr_CurrFilVal_read, taim, Tango::ATTR_VALID, 1,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_GridVal
// 
// description : 	Extract real attribute values for GridVal acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_GridVal(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_GridVal(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
//	vector<bool> values;
	vector<u_short> raw_val;

//	attr_names.push_back("db_50");
	attr_names.push_back("raw_msg_50");
	devattr = devintlk->read_attributes(attr_names);
//	(*devattr)[0] >> values;
//	(*devattr)[1] >> raw_val;
	(*devattr)[0] >> raw_val;
	delete devattr;
*/
#if 0
	unsigned char arr[4]; 
	for(int j=0, k=3;j<4;j++, k--){
		
//		DEBUG_STREAM << j+300 << "-"<< ((Tango::DevUShort)(raw_val.at(j+300))) << "/";
		DEBUG_STREAM << j+300 << "-"<< rawmsg_50[j+300] << "/";
//		bcopy(&(raw_val.at(j+300)), &(arr[k]), 1); 
		bcopy(&(rawmsg_50[j+300]), &(arr[k]), 1); 
	}
	float volt;

	bcopy(&arr[0], &volt, 4);
	DEBUG_STREAM <<"volt:"<< volt << endl;
	grid_val = (double)volt;
#else
	grid_val = rawmsg_10[3];
#endif

	attr_GridVal_read = &grid_val;

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_GridVal_read, taim, Tango::ATTR_VALID, 1,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_GunSafety
// 
// description : 	Extract real attribute values for GunSafety acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_GunSafety(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_GunSafety(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values;

	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values;
	delete devattr;
*/

	for(int j=0;j<GUNSAFETY_DIM;j++){
//		gun_safety[j] = (Tango::DevBoolean)(values.at(pos_gun_safety[j]));
		gun_safety[j] = msg_50[pos_gun_safety[j]];
		DEBUG_STREAM << "read values " << name_gun_safety[j]<< " val: "<< gun_safety[j]<<endl;
	}
	
	attr_GunSafety_read = &gun_safety[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_GunSafety_read, taim, Tango::ATTR_VALID, GUNSAFETY_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_FilAbi
// 
// description : 	Extract real attribute values for FilAbi acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_FilAbi(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_FilAbi(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values;

	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values;
	delete devattr;
*/

	for(int j=0;j<FILABI_DIM;j++){
//		fil_abi[j] = (Tango::DevBoolean)(values.at(pos_fil_abi[j]));
		fil_abi[j] = msg_50[pos_fil_abi[j]];
		DEBUG_STREAM << "read values " << name_fil_abi[j]<< " val: "<< fil_abi[j]<<endl;
	}
	
	attr_FilAbi_read = &fil_abi[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_FilAbi_read, taim, Tango::ATTR_VALID, FILABI_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_FilCurr
// 
// description : 	Extract real attribute values for FilCurr acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_FilCurr(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_FilCurr(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values;

	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values;
	delete devattr;
*/

	for(int j=0;j<FILCURR_DIM;j++){
//		fil_curr[j] = (Tango::DevBoolean)(values.at(pos_fil_curr[j]));
		fil_curr[j] = msg_50[pos_fil_curr[j]];
		DEBUG_STREAM << "read values " << name_fil_curr[j]<< " val: "<< fil_curr[j]<<endl;
	}
	
	attr_FilCurr_read = &fil_curr[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_FilCurr_read, taim, Tango::ATTR_VALID, FILCURR_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_FilReady
// 
// description : 	Extract real attribute values for FilReady acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_FilReady(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_FilReady(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values;

	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values;
	delete devattr;
*/

	for(int j=0;j<FILREADY_DIM;j++){
//		fil_ready[j] = (Tango::DevBoolean)(values.at(pos_fil_ready[j]));
		fil_ready[j] = msg_50[pos_fil_ready[j]];
		DEBUG_STREAM << "read values " << name_fil_ready[j]<< " val: "<< fil_ready[j]<<endl;
	}
	
	attr_FilReady_read = &fil_ready[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_FilReady_read, taim, Tango::ATTR_VALID, FILREADY_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_GunHv
// 
// description : 	Extract real attribute values for GunHv acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_GunHv(Tango::Attribute &attr)
{
/*	DEBUG_STREAM << "IntlkGun::read_GunHv(Tango::Attribute &attr) entering... "<< endl;
	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values;

	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values;
	delete devattr;
*/

	for(int j=0;j<GUNHV_DIM;j++){
//		gun_hv[j] = (Tango::DevBoolean)(values.at(pos_gun_hv[j]));
		gun_hv[j] = msg_50[pos_gun_hv[j]];
		DEBUG_STREAM << "read values " << name_gun_hv[j]<< " val: "<< gun_hv[j]<<endl;
	}
	
	attr_GunHv_read = &gun_hv[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_GunHv_read, taim, Tango::ATTR_VALID, GUNHV_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		IntlkGun::read_AbiTrigGun
// 
// description : 	Extract real attribute values for AbiTrigGun acquisition result.
//
//-----------------------------------------------------------------------------
void IntlkGun::read_AbiTrigGun(Tango::Attribute &attr)
{
	DEBUG_STREAM << "IntlkGun::read_AbiTrigGun(Tango::Attribute &attr) entering... "<< endl;
/*	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<bool> values;

	attr_names.push_back("db_50");
	devattr = devintlk->read_attributes(attr_names);
	(*devattr)[0] >> values;
	delete devattr;
*/

	for(int j=0;j<ABITRIGGUN_DIM;j++){
//		abi_trig_gun[j] = (Tango::DevBoolean)(values.at(pos_abi_trig_gun[j]));
		abi_trig_gun[j] = msg_50[pos_abi_trig_gun[j]];
		DEBUG_STREAM << "read values " << name_abi_trig_gun[j]<< " val: "<< abi_trig_gun[j]<<endl;
	}
	
	attr_AbiTrigGun_read = &abi_trig_gun[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_50 , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;

//	attr.set_value(attr_Db_50_read, MESG_50_LEN_B, 0, false);
//	if(timestamp_50!=0){
		attr.set_value_date_quality(attr_AbiTrigGun_read, taim, Tango::ATTR_VALID, ABITRIGGUN_DIM,0,false);
//	}
//	else{
//		attr.set_value_date_quality(attr_Db_50_read, 0, Tango::ATTR_INVALID, 3,0,false);
//	}
}

//+----------------------------------------------------------------------------
//
// method : 		match_labels(char* dbn)
// 
// description : 	search labels
//
//-----------------------------------------------------------------------------
void IntlkGun::match_labels(const char* dbn)
{
	DEBUG_STREAM << "IntlkGun::match_labels(char* dbn) entering... "<< endl;
	vector<Tango::DeviceAttribute> *devattr;
	vector<string> attr_names;
	vector<string> labels;
	attr_names.push_back(dbn);
	try {
		devattr = devintlk->read_attributes(attr_names);
		(*devattr)[0] >> labels;
		delete devattr;
		set_state(Tango::ON);	
		set_status("Connected to device " + intlk_srv);
		DEBUG_STREAM << "IntlkGun::IntlkGun() Connected to devintlk:"   << intlk_srv   << endl;
	} 
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Failed to connect to devintlk " + intlk_srv);
		ERROR_STREAM <<"IntlkGun::IntlkGun() Failed to connect to devintlk " + intlk_srv << endl;
	}
switch(dbn[8]){
	case '0':
	if(labels.size()!=0){
		string cur_lab;
		for(size_t j=0;j<labels.size();j++){
			cur_lab = (string)(labels.at(j));
			for(int k=0;k<GUNSAFETY_DIM;k++){
				if((cur_lab.find(GunSafetyMask[k]))!=string::npos)pos_gun_safety[k] = j;
			}
			for(int k=0;k<FILABI_DIM;k++){
				if((cur_lab.find(FilAbiMask[k]))!=string::npos)pos_fil_abi[k] = j;
			}
			for(int k=0;k<FILCURR_DIM;k++){
				if((cur_lab.find(FilCurrMask[k]))!=string::npos)pos_fil_curr[k] = j;
			}
			for(int k=0;k<FILREADY_DIM;k++){
				if((cur_lab.find(FilReadyMask[k]))!=string::npos)pos_fil_ready[k] = j;
			}
			for(int k=0;k<GUNHV_DIM;k++){
				if((cur_lab.find(GunHVMask[k]))!=string::npos)pos_gun_hv[k] = j;
			}
			for(int k=0;k<ABITRIGGUN_DIM;k++){
				if((cur_lab.find(AbiTrigGunMask[k]))!=string::npos)pos_abi_trig_gun[k] = j;
			}
		}
	// if names of the signal don't correspond
		bool found_err = false;
		for(int k=0;k<GUNSAFETY_DIM;k++){
			if(pos_gun_safety[k] == -1){
				ERROR_STREAM << "not found pos gun_safety: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<FILABI_DIM;k++){
			if(pos_fil_abi[k] == -1){
				ERROR_STREAM << "not found pos fil_abi: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<FILCURR_DIM;k++){
			if(pos_fil_curr[k] == -1){
				ERROR_STREAM << "not found pos fil_curr: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<FILREADY_DIM;k++){
			if(pos_fil_ready[k] == -1){
				ERROR_STREAM << "not found pos fil_ready: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<GUNHV_DIM;k++){
			if(pos_gun_hv[k] == -1){
				ERROR_STREAM << "not found pos gun_hv: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<ABITRIGGUN_DIM;k++){
			if(pos_abi_trig_gun[k] == -1){
				ERROR_STREAM << "not found pos abi_trig_gun: " << k<< endl;
				found_err = true;
			}
		}
		if(found_err){
			ERROR_STREAM << "signal names not corresponding db 50" << endl;
			set_state(Tango::FAULT);
			set_status("signal names not corresponding ");
		}
	}
	break;
	case '3':
	if(labels.size()!=0){
		string cur_lab;
		for(size_t j=0;j<labels.size();j++){
			cur_lab = (string)(labels.at(j));
			for(int k=0;k<GUNSAFETY_DIM;k++){
				if((cur_lab.find(GunSafetyMask[k]))!=string::npos)pos_alm_gun_safety[k] = j;
			}
			for(int k=0;k<FILABI_DIM;k++){
				if((cur_lab.find(FilAbiMask[k]))!=string::npos)pos_alm_fil_abi[k] = j;
			}
			for(int k=0;k<FILCURR_DIM;k++){
				if((cur_lab.find(FilCurrMask[k]))!=string::npos)pos_alm_fil_curr[k] = j;
			}
			for(int k=0;k<FILREADY_DIM;k++){
				if((cur_lab.find(FilReadyMask[k]))!=string::npos)pos_alm_fil_ready[k] = j;
			}
			for(int k=0;k<GUNHV_DIM;k++){
				if((cur_lab.find(GunHVMask[k]))!=string::npos)pos_alm_gun_hv[k] = j;
			}
		}
	// if names of the signal don't correspond
		bool found_err = false;
		for(int k=0;k<GUNSAFETY_DIM;k++){
			if(pos_alm_gun_safety[k] == -1){
				ERROR_STREAM << "not found pos alm_gun_safety: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<FILABI_DIM;k++){
			if((pos_alm_fil_abi[k] == -1)&&(k!=0)){
				ERROR_STREAM << "not found pos alm_fil_abi: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<FILCURR_DIM;k++){
			if((pos_alm_fil_curr[k] == -1)&&(k!=0)){
				ERROR_STREAM << "not found pos alm_fil_curr: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<FILREADY_DIM;k++){
			if((pos_alm_fil_ready[k] == -1)&&(k!=0)){
				ERROR_STREAM << "not found pos alm_fil_ready: " << k<< endl;
				found_err = true;
			}
		}
		for(int k=0;k<GUNHV_DIM;k++){
			if((pos_alm_gun_hv[k] == -1)&&(k!=0)){
				ERROR_STREAM << "not found pos alm_gun_hv: " << k<< endl;
				found_err = true;
			}
		}
		if(found_err){
			ERROR_STREAM<<"signal names not corresponding db 53"<<endl;
			set_state(Tango::FAULT);
			set_status("signal names not corresponding ");
		}
	}
	break;
}

}



//+------------------------------------------------------------------
/**
 *	method:	IntlkGun::acknowledge_all
 *
 *	description:	method to execute "Acknowledge_all"
 *
 *
 */
//+------------------------------------------------------------------
void IntlkGun::acknowledge_all()
{
	DEBUG_STREAM << "IntlkGun::acknowledge_all(): entering... !" << endl;

	//	Add your own code to control device here
	//	Add your own code to control device here
	Tango::DevVarLongArray indxs;
//	indxs.svalue.length(0);
	indxs.length(23);
	int count = 0;

	for(int k=0;k<GUNSAFETY_DIM;k++){
		if(pos_alm_gun_safety[k] != -1){
			indxs[count] = pos_alm_gun_safety[k];
			count++;
		}
	}
	for(int k=0;k<FILABI_DIM;k++){
		if((pos_alm_fil_abi[k] !=-1)&&(k!=0)){
			indxs[count] = pos_alm_fil_abi[k];
			count++;
		}
	}
	for(int k=0;k<FILCURR_DIM;k++){
		if((pos_alm_fil_curr[k] != -1)&&(k!=0)){
			indxs[count] = pos_alm_fil_curr[k];
			count++;
		}
	}
	for(int k=0;k<FILREADY_DIM;k++){
		if((pos_alm_fil_ready[k] != -1)&&(k!=0)){
			indxs[count] = pos_alm_fil_ready[k];
			count++;
		}
	}
	for(int k=0;k<GUNHV_DIM;k++){
		if((pos_alm_gun_hv[k] != -1)&&(k!=0)){
			indxs[count] = pos_alm_gun_hv[k];
			count++;
		}
	}
	for(int k=0;k<ABITRIGGUN_DIM;k++){
		if((pos_alm_abi_trig_gun[k] != -1)&&(k!=0)){
			indxs[count] = pos_alm_abi_trig_gun[k];
			count++;
		}
	}
//ack selected on all gun signals
	DEBUG_STREAM << "ack to " <<count << " signals " << endl; 	

	Tango::DeviceData Din;
	Din << indxs;
	
	try {
//		devintlk->command_inout("Acknowledge_all");
		devintlk->command_inout("Ack_selected", Din);
		set_state(Tango::ON);
		set_status("command  ack sent " );
	}
	catch(Tango::DevFailed e) {	
		set_state(Tango::FAULT);
		set_status("Cannot send command  ack " );			
		ERROR_STREAM << "Cannot send command  ack"  << endl; 	
	}


}

}	//	namespace
